﻿/** Classe AbulEduApplicationV1
  *
  * Classe de base en version une pour les applications AbulEdu permettant de structurer
  * nos logiciels et respecter ainsi un certain nombre de choses dont l'interface publique
  * à laquelle le plugin peut se connecter pour échanger des données (logs pour le moment)
  *
  * @see https://redmine.ryxeo.com/projects/ryxeo/wiki/AbulEduApplicationV1
  * @author 2009-2012 Eric Seigne <eric.seigne@ryxeo.com>
  * @author 2011 Jean-Louis Frucot <frucot.jeanlouis@free.fr>
  * @see The GNU Public License (GPL)
  *
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
  * for more details.
  *
  * You should have received a copy of the GNU General Public License along
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  */

#include "abuleduapplicationv1.h"

void noDebugOutput(QtMsgType type, const char *msg)
{
    switch (type) {
    case QtDebugMsg:
        break;
    case QtWarningMsg:
        break;
    case QtCriticalMsg:
        fprintf(stderr,"%s",msg);
        fflush(stderr);
        break;
    case QtFatalMsg:
        fprintf(stderr,"%s",msg);
        fflush(stderr);
        abort();
    }
    return;
}

void debugOutput(QtMsgType type, const char *msg)
{
    QString logMessage;

    switch (type) {
    case QtDebugMsg:
        logMessage = msg;
        break;
    case QtWarningMsg:
        logMessage = msg;
        break;
    case QtCriticalMsg:
        logMessage = msg;
        break;
    case QtFatalMsg:
        logMessage = msg;
        abort();
    }

    if(logMessage != "") {

        //Si on est sous windows
#ifdef Q_OS_WIN32
        //et qu'on est en mode debug on veut avoir le retour dans le qt debug "sortie de l'appli"
#ifndef QT_NO_DEBUG
        fprintf(stderr,"%s\n",logMessage.toLatin1().constData());
        fflush(stderr);
#else
        QFile fileLog(QDir::tempPath() + "/" + abeApp->applicationName() + ".log");
        QFileInfo fi(fileLog);
        if(fileLog.open(QIODevice::Append | QIODevice::Text)) {
            fileLog.write(logMessage.toLatin1() + "\n");
            fileLog.close();
        }
        else {
            QMessageBox msgBox;
            msgBox.setText(QString("Erreur d'ouverture du fichier log : %1").arg(fi.absoluteFilePath()));
            msgBox.exec();
        }
#endif
#else
        fprintf(stderr,"%s\n",logMessage.toLatin1().constData());
#endif
    }
}

/** styles speciaux pour tablettes, en particulier les scrollbar qu'on double de largeur (au moins) */
class MyTabletStyle : public QProxyStyle
{
  public:
    int pixelMetric(PixelMetric metric, const QStyleOption* option, const QWidget* widget) const
    {
        if (metric == PM_ScrollBarExtent)
            return 45;
        return QProxyStyle::pixelMetric(metric, option, widget);
    }
    QSize sizeFromContents(QStyle::ContentsType type, const QStyleOption* option, const QSize& contentsSize, const QWidget* widget) const
    {
        if (type == QStyle::CT_ScrollBar)
            return contentsSize;
        return QProxyStyle::sizeFromContents(type, option, contentsSize, widget);
    }
};

/** Recuperation pour les gestures */
bool qt_sendSpontaneousEvent(QObject *receiver, QEvent *event)
{
    return QCoreApplication::sendSpontaneousEvent(receiver, event);
}

/** constructeur ... */
AbulEduApplicationV1::AbulEduApplicationV1(int &argc, char *argv[], QString appName, QString appVersion, QString organizationDomain, QString organizationName):
    QtSingleApplication(argc, argv),
    m_sharedMemory("abeUpdateRequest")
{
    //Si on compile en mode debug, on ne change pas le comportement par defaut et historique
#ifndef QT_NO_DEBUG
    qInstallMsgHandler(debugOutput);
#else
    //Si on compile en mode release on se garde une super option --debug qui permet de passer une appli en prod en mode debug
    QStringList parametres = qApp->arguments(); //récupère la liste des paramètres
    if(parametres.contains("--debug")) {
        qInstallMsgHandler(debugOutput);
    }
    else  {
        qInstallMsgHandler(noDebugOutput);
    }
#endif

    m_localDebug = DEBUG_ABULEDUAPPLICATIONV1;
    if(m_localDebug) {
        qDebug() << "AbulEduApplicationV1: constructeur";
    }

    m_abuleduVirtualKeyboard = new AbulEduVirtualKeyboardV1();
    //Attention, provoque le bug des accents impossibles ... ê
    setInputContext(m_abuleduVirtualKeyboard);

    //Si on est en mode tablette on cache le curseur de la souris
#ifdef __ABULEDUTABLETTEV1__MODE__
    QApplication::setOverrideCursor(QCursor(Qt::BlankCursor));
    setStyle(new MyTabletStyle);
    m_abuleduVirtualKeyboard->abeSetEnabled(true);
#endif

    setApplicationName(appName);
    setApplicationVersion(appVersion);
    setOrganizationName(organizationName);
    setOrganizationDomain(organizationDomain);
    QFontDatabase andika;
    andika.addApplicationFont(":/abuleduapplicationv1/fonts/andika");
    setFont(QFont("andika"));

    if(organizationName == "")
        qWarning() << "L'appel au constructeur AbulEduApplicationV1 ne specifie pas d'organizationName ... le chargement de plugins personnels ne sera pas automatique.";

    //Initialisation des différentes variables
    m_applicationLongName  = "";
    m_interfaceLang = QLocale::system().name().section('_', 0, 0);
    m_username = "";
    p_setUserName(); //on affecte la variable m_username dans cette fonction qui gere les OS
    m_decorRatio       = 1;
    int already_requested = 0;

    m_nam = new AbulEduNetworkAccessManagerV1(this);
    m_identite = new AbulEduIdentitesV1(this);
    m_identite->setObjectName("AbulEduApplicationV1/AbulEduIdentitesV1");

    //pas la peine d'aller plus loin si le developpeur n'a pas fait son boulot !
    if(applicationName() == "" || applicationVersion() == "") {
        qWarning() << trUtf8("AbulEduApplicationV1:: Attention: applicationName ou applicationVersion non défini !!!");
        qWarning() << applicationName() << "::" << applicationVersion();
        //on essaye tout de meme de charger les plugins
        loadPlugins();
        return;
    }
    // ============================================================================== memoire partagee et requete reseau update
    if (m_sharedMemory.isAttached())
        detach();
    //creation du segment de memoire partagee
    if(m_sharedMemory.create(sizeof(already_requested))) {
        if(m_localDebug)
            qDebug() << "AbulEduApplicationV1 : creation de la zone memoire partagee, taille" << m_sharedMemory.size();

        // ----------------------------------
        //On cherche pour voir s'il y a une mise a jour de l'application de disponible
        //On lance notre requete reseau et on gere la suite

        //Il faudrait pouvoir lancer plusieurs requetes en // sur differents supports
        //avec ou sans proxy, en http et en https, voir meme sur d'autres domaines
        QUrl urlUpdates("http://updates.ryxeo.com/application/" + applicationName() + "/version/" + applicationVersion() + "/os/" + os() + "/dist/" + linuxDistrib() + "/format/xml/uid/" + uniqueUID() + "/age/" + QString::number(execAge()).toLatin1().toBase64() + "/");
        connect(m_nam, SIGNAL(finished(QNetworkReply*)),this, SLOT(slotOnlineUpdateRequest(QNetworkReply*)), Qt::UniqueConnection);
        m_networkReply = m_nam->get(QNetworkRequest(urlUpdates));
        m_networkReply->setObjectName("OnlineUpdateRequest");
        connect(m_networkReply, SIGNAL(error(QNetworkReply::NetworkError)), this, SLOT(slotReplyError(QNetworkReply::NetworkError)), Qt::UniqueConnection);
        // ----------------------------------


        // on écrit le flag comme quoi la requete reseau est deja partie ... merci de ne pas relancer cette requete :)
        m_sharedMemory.lock();
        already_requested = 1;
        if(m_localDebug)
            qDebug() << "AbulEduApplicationV1 : ecriture dans la memoire partagee: requete reseau lancee";
        memcpy(m_sharedMemory.data(), &already_requested, sizeof(already_requested));
        m_sharedMemory.unlock();
    }
    else {
        if(m_localDebug)
            qDebug() << "AbulEduApplicationV1 : zone memoire partagee deja existante, la requete reseau a deja ete lancee";
        // si le segment existe deja on l'utilise
        if(m_sharedMemory.error() == QSharedMemory::AlreadyExists) {
            if(m_sharedMemory.attach()) {
                // lecture des donnees
                m_sharedMemory.lock();
                already_requested = *(bool *)m_sharedMemory.constData();
                m_sharedMemory.unlock();
            }
            else {
                if(m_localDebug)
                    qDebug() << "AbulEduApplicationV1 : erreur sur sharedMemory :: " << m_sharedMemory.errorString();
            }
        }
        else {
            if(m_localDebug)
                qDebug() << "AbulEduApplicationV1 : erreur sur sharedMemory :: " << m_sharedMemory.errorString();
        }
    }
    // ============================================================================== fin de gestion memoire partagee et requete reseau update


    // ============================================================================== logiciel de plus de six mois
    QFileInfo fi(qApp->applicationFilePath());
    QDateTime ageDuFichier(fi.created());
    QDateTime agePlusSixMois(ageDuFichier.addMonths(24));
    QUrl urlSixMois("http://updates.ryxeo.com/application/" + applicationName() + "/version/" + applicationVersion() + "/os/" + os() + "/dist/" + linuxDistrib() + "/format/xml/uid/" + uniqueUID() + "/age/" + QString::number(execAge()).toLatin1().toBase64() + "/");
    //Si l'application a plus de six mois ... on ouvre un browser sur la page speciale
    //pour s'assurer que des très vieux logiciels ne sont plus utilisés
    //passé a 24 mois, on reviendra sur l'idée de 6 mois mais ça fait quand meme un peu court 6 mois, 2 ans c'est déjà
    //un peu plus compréhensible
    if(agePlusSixMois < QDateTime::currentDateTime()) {
        AbulEduNetworkThreadV1 *browser = new AbulEduNetworkThreadV1(urlSixMois);
        browser->start();
        m_browserLaunched = true;
    }
    // ============================================================================== fin logiciel de plus de six mois

    // ============================================================================== plus de x lancements sans reponse de serveur

    // ============================================================================== fin plus de x lancements sans reponse de serveur
    if(m_localDebug)
        qDebug() << "AbulEduApplicationV1 : avant loadPlugins";
    loadPlugins();
    if(m_localDebug)
        qDebug() << "AbulEduApplicationV1 : après loadPlugins";
}

AbulEduApplicationV1::~AbulEduApplicationV1()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }

    //On detache la memoire partagee
    detach();
}

void AbulEduApplicationV1::loadPlugins()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    // ---------------------------------- chargement plugin
    //Ensuite on charge les plugins qui se trouvent dans le sous dossier plugins de l'application
    QDir pluginsDir(applicationDirPath());
    if (pluginsDir.dirName().toLower() == "debug" || pluginsDir.dirName().toLower() == "release")
        pluginsDir.cdUp();
#if defined(Q_OS_MAC)
    if (pluginsDir.dirName() == "MacOS") {
        pluginsDir.cdUp();
        pluginsDir.cdUp();
        pluginsDir.cdUp();
    }
#endif
    if(pluginsDir.cd("plugins")) {
        foreach (QString fileName, pluginsDir.entryList(QDir::Files)) {
            if(m_localDebug)
                qDebug() << "AbulEduApplicationV1 : Chargement du plugin " << fileName;
            QPluginLoader pluginLoader(pluginsDir.absoluteFilePath(fileName));
            QObject *plugin = pluginLoader.instance();
            if(m_localDebug)
                qDebug() << "AbulEduApplicationV1 : Chargement du plugin (2)" << plugin;
            if (plugin && m_localDebug) {
                qDebug() << "AbulEduApplicationV1 : Chargement du plugin " << fileName << " réussi";
            }
        }
    }
    // ---------------------------------- Fin du chargement plugin

    // ---------------------------------- chargement plugin
    //Puis les plugins qui se trouve dans le repertoire de l'utilisateur (permet de mises a jours "douces")
#ifdef Q_OS_WIN32
    QSettings settings(QSettings::UserScope, "Microsoft", "Windows");
    settings.beginGroup("CurrentVersion/Explorer/Shell Folders");
    m_homepath = settings.value("Personal").toString();
#else
    m_homepath = QDir::homePath();
#endif
    QString pluginsDirPerso(m_homepath + "/" + organizationName() + "/" + applicationName() + "/plugins");

    if(m_localDebug)
        qDebug() << "AbulEduApplicationV1 : Chargement des plugins perso " << pluginsDirPerso;
    if(pluginsDir.cd(pluginsDirPerso)) {
        foreach (QString fileName, pluginsDir.entryList(QDir::Files)) {
            if(m_localDebug)
                qDebug() << "AbulEduApplicationV1 : Chargement du plugin personnel " << fileName;
            QPluginLoader pluginLoader(pluginsDir.absoluteFilePath(fileName));
            QObject *plugin = pluginLoader.instance();
            if(m_localDebug)
                qDebug() << "AbulEduApplicationV1 : Chargement du plugin personnel (2)" << plugin;
            if (plugin && m_localDebug) {
                qDebug() << "AbulEduApplicationV1 : Chargement du plugin personnel " << fileName << " réussi";
            }
        }
    }
    // ---------------------------------- Fin du chargement plugin
}

void AbulEduApplicationV1::linuxAnalyse()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }

    QStringList dists_debian;
    QStringList dists_ubuntu_lts;
    QStringList dists_ubuntu;
    QStringList dists_fedora;
    QStringList dists_mageia;

    dists_debian     << "squeeze" << "wheezy" << "sid";
    dists_ubuntu_lts << "dapper"  << "hardy"  << "lucid"  << "precise";
    dists_ubuntu     << "warty"   << "hoary"  << "breezy" << "edgy"    << "feisty" << "gutsy" << "intrepid" << "jaunty" << "karmik" << "maverick" << "natty" << "oneiric" << "quantal" << "raring";
    dists_mageia     << "thornicroft";

    //Signature des fedora
    QFile fedora("/etc/fedora-release");

    //Pour l'instant c'est uniquement ubuntu qui propose ca
    //Et mageia 2 aussi ...
    QFile lsb("/etc/lsb-release");
    if(lsb.exists()) {
        if (lsb.open(QIODevice::ReadOnly | QIODevice::Text)) {
            QRegExp release("DISTRIB_CODENAME=(.*)");
            while (!lsb.atEnd() && m_linuxDistrib == "") {
                QByteArray line = lsb.readLine();
                release.indexIn(line.trimmed());
                if(release.cap(1) != "" && m_localDebug) {
                    qDebug() << "AbulEduApplicationV1::linuxPackageFormat On a trouve le LSB :: " << line << " ici : " << release.cap(1).trimmed();
                }
                if(dists_debian.contains(release.cap(1)) || dists_ubuntu.contains(release.cap(1)) || dists_ubuntu_lts.contains(release.cap(1))) {
                    m_linuxDistrib = release.cap(1).trimmed();
                    m_linuxPackageFormat = "deb";
                }
                if(dists_mageia.contains(release.cap(1))) {
                    m_linuxDistrib = release.cap(1).trimmed();
                    m_linuxPackageFormat = "rpm";
                }

            }
            lsb.close();
        }
    }
    else if(fedora.exists()) {
        m_linuxPackageFormat = "rpm";
    }
    else {
        if(m_localDebug)
            qDebug() << "Lancer la commande 'lsb_release -a";
    }

    if(m_linuxPackageFormat == "" || m_linuxDistrib == "") {
        qWarning() << trUtf8("La distribution ou le format de paquet ne sont pas connus ... merci de nous contacter pour qu'on puisse ajouter votre systeme dans notre liste: devteam@ryxeo.com");
    }
}

QString AbulEduApplicationV1::os()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }

    QString os("");
#if defined(Q_OS_MAC)
    os="osx";
#elif defined(Q_OS_WIN32)
    os="windows";
#elif defined(Q_OS_LINUX)
    QString arch(""); //pour l'instant i386 / amd64
    //L'architecture ... pour l'instant on ne fait pas les ARM / Sparc etc.
#ifdef __x86_64__
    arch = "amd64";
#else
    arch = "i386";
#endif

    //On regroupe tout ca linux-deb-lucid-amd64
    os="linux-" + linuxDistrib() + "-" + linuxPackageFormat() + "-" + arch;
#endif
    return os;
}

QString AbulEduApplicationV1::uniqueUID()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }

    QByteArray retour("000102030405");
    foreach(QNetworkInterface interface, QNetworkInterface::allInterfaces()) {
        if (interface.flags().testFlag(QNetworkInterface::IsUp) && !interface.flags().testFlag(QNetworkInterface::IsLoopBack)) {
            foreach (QNetworkAddressEntry entry, interface.addressEntries()) {
                if ( interface.hardwareAddress() != "00:00:00:00:00:00" && entry.ip().toString().contains(".")) {
                    retour.clear();
                    retour.append(interface.hardwareAddress().remove(":").toLower());
                }
            }
        }
    }
    return retour.toHex().toBase64();
}

QString AbulEduApplicationV1::getAbeUserLogin()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }

    return m_username;
}

int AbulEduApplicationV1::execAge()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }

    QFileInfo fi(qApp->applicationFilePath());
    int ageDuFichier(fi.created().daysTo(QDateTime::currentDateTime()));
    if(m_localDebug)
        qDebug() << "Age du fichier :: " << ageDuFichier;
    return ageDuFichier;
}

QString AbulEduApplicationV1::linuxDistrib()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }

    if(m_linuxDistrib == "")
        linuxAnalyse();
    return m_linuxDistrib;
}

QString AbulEduApplicationV1::linuxPackageFormat()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }

    if(m_linuxPackageFormat == "")
        linuxAnalyse();
    return m_linuxPackageFormat;
}

void AbulEduApplicationV1::setAbeApplicationDebugLevel(int i)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }

    m_localDebug = i;
}

void AbulEduApplicationV1::setAbeApplicationLongName(QString s)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }

    m_applicationLongName = s;
}

QString AbulEduApplicationV1::getAbeApplicationInterfaceLang()
{
    return m_interfaceLang;
}

void AbulEduApplicationV1::setAbeApplicationInterfaceLang(QString lang)
{
    m_interfaceLang = lang;
}

QString AbulEduApplicationV1::getAbeApplicationLongName()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }

    return(m_applicationLongName);
}

void AbulEduApplicationV1::slotOnlineUpdateRequest(QNetworkReply *reply)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }

    //Si ce n'est pas pour moi je droppe
    if(reply->objectName() != "OnlineUpdateRequest") {
        return;
    }

    disconnect(m_nam, SIGNAL(finished(QNetworkReply*)),this, SLOT(slotOnlineUpdateRequest(QNetworkReply*)));

    // Pas d'erreur ?
    if (reply->error() == QNetworkReply::NoError) {
        int v = reply->attribute(QNetworkRequest::HttpStatusCodeAttribute).toInt();
        if (v >= 200 && v < 300) // Success
        {
            QString titre("");
            QString desc("");
            QString link("");
            QString lang("");
            QDomDocument *xml = new QDomDocument();
            xml->setContent(reply->readAll());
            if(m_localDebug)
                qDebug() << "AbulEduApplicationV1 : download XML update " << xml->toString();

            QDomElement xmlElem = xml->documentElement();
            QDomNode node = xmlElem.firstChild();
            while(!node.isNull()) {
                QDomElement element = node.toElement(); // On essaie de convertir le node en element
                if(!element.isNull()) {
                    // Le node est bien un élément.
                    if(element.tagName()=="application") {
                        QDomNode node=element.firstChild(); //On parcourt le document à partir du tag "application"
                        while(!node.isNull()) {
                            QDomElement elem=node.toElement(); //On cherche le titre du "Channel"
                            if(elem.tagName()=="title") {
                                titre = elem.text();
                            }
                            else if((elem.tagName()=="link")) {
                                link = elem.text();
                            }
                            else if((elem.tagName()=="description")) {
                                desc = elem.text();
                            }
                            else if((elem.tagName()=="language")) {
                                lang = elem.text();
                            }
                            node=node.nextSibling();
                        }
                    }
                    node = node.nextSibling();
                }
            }

            //On affiche une boite d'information (popup) à l'utilisateur ...
            if(titre.length()) {
                AbulEduMessageBoxV1* msgBox = new AbulEduMessageBoxV1(titre, desc);
                msgBox->show();
                //Quand on ferme la fenetre on lance le signal END -- evite une collision entre le splascreen et une popup de mise a jour disponible
                connect(msgBox, SIGNAL(destroyed()), this, SIGNAL(signalApplicationMessageEnd()),Qt::UniqueConnection);
                //fenetre de message ouverte, on informe tout le monde
                emit signalApplicationMessageStart();
            }
            delete xml;
        }
        //S'il s'agit de redirections ...
        else if (v >= 300 && v < 400)
        {
            // Get the redirection url
            QUrl newUrl = reply->attribute(QNetworkRequest::RedirectionTargetAttribute).toUrl();
            // Because the redirection url can be relative,
            // we have to use the previous one to resolve it
            newUrl = reply->url().resolved(newUrl);
            QNetworkRequest redirection(newUrl);
            m_networkReply = m_nam->get(redirection);
            connect(m_networkReply, SIGNAL(error(QNetworkReply::NetworkError)), this, SLOT(slotReplyError(QNetworkReply::NetworkError)),Qt::UniqueConnection);
            return; // to keep the manager for the next request
        }
    }
    reply->deleteLater();
}

void AbulEduApplicationV1::slotReplyError(QNetworkReply::NetworkError code)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
        qDebug() << "  " << code;
    }
    m_networkReply->deleteLater();
}

void AbulEduApplicationV1::p_setUserName()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__ << m_username << " (start) " << objectName();
    }
#ifdef Q_WS_WIN
    m_username = getenv("USER");
    if(m_username == "") {
        m_username = getenv("USERNAME");
    }
#endif
#ifdef Q_WS_X11
    m_username = getenv("USER");
    if(m_username == "") {
        m_username = getenv("USERNAME");
    }
#endif
#ifdef Q_WS_MAC
    m_username = getenv("USER");
    if(m_username == "") {
        m_username = getenv("USERNAME");
    }
#endif
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__ << m_username << " (end) " << objectName();
    }
}

void AbulEduApplicationV1::detach()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    if (!m_sharedMemory.detach()) {
        if(m_localDebug)
            qDebug() << "  Unable to detach from shared memory.";
    }
}

void AbulEduApplicationV1::setAbeApplicationDecorRatio(float ratio)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    m_decorRatio = ratio;
}

float AbulEduApplicationV1::getAbeApplicationDecorRatio()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    return m_decorRatio;
}

AbulEduNetworkAccessManagerV1 *AbulEduApplicationV1::getAbeNetworkAccessManager()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    return m_nam;
}

AbulEduIdentitesV1 *AbulEduApplicationV1::getAbeIdentite()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    return m_identite;
}

QString AbulEduApplicationV1::getAbeHomepath()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    return m_homepath;
}

void AbulEduApplicationV1::sendSignalDownloaded(QString s)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    emit signalDownloaded(s);
}

void AbulEduApplicationV1::setAbeApplicationVirtualKeyboard(bool enabled)
{
    m_abuleduVirtualKeyboard->abeSetEnabled(enabled);
}
