/** Classe AbulEduFileV1
  * @see https://redmine.ryxeo.com/projects/
  * @author 2010-2013 Eric Seigne <eric.seigne@ryxeo.com>
  * @author 2012 Icham Sirat <icham.sirat@ryxeo.com>
  * @see The GNU Public License (GNU/GPL) v3
  *
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
  * for more details.
  *
  * You should have received a copy of the GNU General Public License along
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  */


#ifndef ABULEDUFILEV1_H
#define ABULEDUFILEV1_H

#ifndef DEBUG_ABULEDUFILEV1
#define DEBUG_ABULEDUFILEV1 0
#endif

#include <QTemporaryFile>
#include <QDomDocument>
#include <QMessageBox>
#include <QDateTime>
#include <QFileInfo>
#include <QPainter>
#include <QObject>
#include <QDebug>
#include <QFile>
#include <QDir>
#include <QtSvg/QSvgRenderer>


#ifdef __WITHEXIV2__
#include <exiv2/image.hpp>
#include <exiv2/exif.hpp>
#endif

#include "quazip.h"
#include "quazipfile.h"
#include "abuleduapplicationv1.h"
#include "abuleduscolomfrv1.h"
#include "abuledumessageboxv1.h"
#include "abuledumimetypev1.h"

class AbulEduFileV1 : public QObject
{
    Q_OBJECT

public:
    explicit AbulEduFileV1(QObject *parent = 0);
    ~AbulEduFileV1();

    /**
      * Cette méthode retourne un identifiant unique pour le fichier temporaire
      * Elle est implémentée pour pallier les risques de sécurité liés au fichier codé en "dur"
      * Utilisation d'un QTemporyFile afin d'avoir un nom unique
      * C'est un fichier créé avec un id imprévisible
      * Je me sers de cet id pour renommer le dossier temp dont j'ai besoin =)
      * note -> le file temp créé s'efface en même tps que la destruction de l'objet
      */
    static QString abeUniqIDTemp();
    /** @obsolete, claque un qwarning
      */
    static QString uniqIDTemp();


    /** Méthode qui redimensionne une image et la copie dans le repertoire destination sous le nom original.jpg
      * @warning : l'extension du fichier de destination est .jpg
      * @brief  Les formats supportés sont : "jpeg", "bmp", "png", "ppm", "xbm", "xpm" et "svg".
      *         Cependant, dans le cas du svg et du png, l'image retournée aura un fond blanc.
      *         Dans le cas des autres formats, l'image sera redimensionnée au profit de la largeur (@see QPixmap::scaledToWidth()).
      * @param QFileInfo *fileInfoImage , l'image d'origine a redimensionnée
      * @param int largeurMaxSouhaitee  , la largeur maximum souhaitée pour le redimensionnement
      * @param QString cheminDestination, la destination de l'image redimensionnée (le résultat)
      * @return bool, true si réussi, false si échec
      */
    bool resizeImage(QFileInfo *fileInfoImage, int largeurMaxSouhaitee, QString cheminDestination);


    /** Sauvegarde au format .abe -> tentative de convegence vers le format OpenDocument
      *
      * cf https://redmine.ryxeo.com/projects/ryxeo/wiki/Format_de_fichier_commun
      *
      * format = zip -> fichier zip tout con
      * format = abe -> fichier abe de type opendocument si on y arrive
      * destFileName = nom complet du fichier de destination (répertoire y compris, nom absolu)
      * fileList = liste des noms de fichiers a inclure dans l'archive (noms absolu)
      * fileBase = base commune de tous les fichiers a inclure (ce qui sera pris pour racine commune, nom absolu)
      * Exemple
      *  fileList = /home/toto/leterrier/logiciel/projet/conf/fichier1, /home/toto/leterrier/logiciel/projet/data/fichierA, /home/toto/leterrier/logiciel/projet/data/fichierB
      *  fileBase = /home/toto/leterrier/logiciel/projet
      *  destFileName = /home/toto/mon_exercice.abe
      */
    bool abeFileSave(QString destFileName, QStringList fileList, QString fileBase = "", QString fileFormat = "zip");

    /** Prépare la sauvegarde, le pointeur abuledufilev1 sera ensuite passé à abuleduboxfilemanager qui fera "l'export" et écrira le fichier
      * soit sur le disque, soit sur la box, soit ...
      *
      * cf https://redmine.ryxeo.com/projects/ryxeo/wiki/Format_de_fichier_commun
      *
      * format = zip -> fichier zip tout con
      * format = abe -> fichier abe de type opendocument si on y arrive
      * fileList = liste des noms de fichiers a inclure dans l'archive (noms absolu)
      * fileBase = base commune de tous les fichiers a inclure (ce qui sera pris pour racine commune, nom absolu)
      * Exemple
      *  fileList = /home/toto/leterrier/logiciel/projet/conf/fichier1, /home/toto/leterrier/logiciel/projet/data/fichierA, /home/toto/leterrier/logiciel/projet/data/fichierB
      *  fileBase = /home/toto/leterrier/logiciel/projet
      */
    bool abeFileExportPrepare(QStringList fileList, QString fileBase = "", QString fileFormat = "zip");

    /** Export le fichier preparé à l'étape précédente
      * sera probablement utilisé par abuleduboxfilemanager quand l'utilisateur aura décidé du nom du fichier
      */
    bool abeFileExportFile(QString destFileName);

    /** Ouverture d'un fichier
      * format = zip -> fichier zip tout con, on l'extrait dans le repertoire dirDest
      * format = abe -> fichier abe de type opendocument si on y arrive -> traitement a definir
      */
    bool abeFileOpen(QString srcFile, QString fileFormat = "zip");
    bool abeFileOpen(QString srcFile, QDir *dest, QString fileFormat = "zip");

    /** Affectation du nom du fichier, sans l'ouvrir
      */
    void abeFileSetFilename(QString filename);

    // =============================================================================================================

    /** Copie les meta data d'un fichier abb vers un fichier abe
      * exemple : m_abuleduFile->abeFileAddMetaDataFromABB(ui->mediathequeGetABB->abeGetLOM(), ui->mediathequeGetABB->abeGetFile()->abeFileGetContent(0).baseName());
      *
      */
    bool abeFileAddMetaDataFromABB(AbulEduSCOLOMFrV1 *abbFile, QString abbFileName);

    // ======================================================================== SETteur
    /** La langue du contenu intellectuel de la ressource, fr-fr, en-uk */
    void abeFileSetLanguage(QString lang);
    /** L'entité principalement responsable de la création du contenu de la ressource */
    void abeFileSetCreator(QString creator);
    /** Une date associée avec un événement dans le cycle de vie de la ressource au format ISO8601 */
    void abeFileSetDate(QString date);
    /** Typiquement, le sujet sera décrit par un ensemble de mots-clefs ou de phrases ou un code de classification qui précisent le sujet */
    void abeFileSetSubject(QString subject);
    /** Typiquement, un titre sera le nom par lequel la ressource est officiellement connue */
    void abeFileSetTitle(QString title);
    /** Typiquement, un élément Droits contiendra un état du droit à gérer une ressource, ou la reférence à un service fournissant cette information.*/
    void abeFileSetRights(QString rights);
    /** Une Description peut contenir, mais ce n'est pas limitatif un résumé, une table des matières, une référence à une représentation graphique du contenu, ou un texte libre */
    void abeFileSetDescription(QString description);
    /** Exemples d'Editeurs incluent une personne, une organisation, ou un service. Typiquement, le nom d'une maison d'édition*/
    void abeFileSetPublisher(QString publisher);
    /** Une entité qui a contribué à la création du contenu*/
    void abeFileSetContributor(QString contributor);
    /** La liste des mots clés */
    void abeFileSetKeywords(QString keywords);
    /** Le type inclut des termes décrivant des catégories, fonctions ou genres généraux pour le contenu, ou des niveaux d'agrégation, Il est recommandé de choisir la valeur du type dans une liste de valeurs
      * @see http://dublincore.org/documents/resource-typelist/
      */
    void abeFileSetType(QString type);
    /** Typiquement, Format peut inclure le media ou les dimensions de la ressource, Exemples de dimensions incluent la taille et la durée. Il est recommandé de choisir la valeur du format dans une liste (ex.mime)
      * @see http://www.iana.org/assignments/media-types/index.html
      */
    void abeFileSetFormat(QString format);
    /** Il est recommandé d'identifier la ressource par une chaîne de caractère ou un nombre conforme à un sytème formel d'identification, ex. ISBN, ISSN, URL */
    void abeFileSetIdentifier(QString identifier);
    /** La ressource actuelle peut avoir été dérivée d'une autre ressource source, en totalité ou en partie.  */
    void abeFileSetSource(QString source);
    /** Une référence à une autre ressource qui a un rapport avec cette ressource */
    void abeFileSetRelation(QString relation);
    /** La portée ou la couverture spatio-temporelle de la ressource. Il est recommandé de choisir la  valeur de Couverture dans un vocabulaire contrôlé (par exemple, un thésaurus de noms géographiques, comme[TGN]) */
    void abeFileSetCoverage(QString coverage);

    /** Permet d'affecter des tags EXIF (pour les images)
      * @param file le fichier pour lequel il faut modifier les tags exif
      * @param artiste le nom de l'auteur
      * @param copyright la licence utilisée
      * @uniqID l'identifiant unique de type http://data.abuledu.org/URI/47f422e0 LIMITE a 128 caracteres, on supprime donc tout le nom 'texte' pour ne garder que la clé
      */
    static bool abeFileSetExiv2Tags(QFileInfo file, QString artist, QString copyright, QString uniqID);

    /** Copie tous les tags EXIF d'un fichier vers un autre
      */
    static bool abeFileCopyExiv2Tags(QString src, QString dst);

    /** Permet d'avoir en retour un format de fichier bien formatté en Mo / Ko etc. */
    static QString abeFileFormatSize(qint64 num);

    // ======================================================================== GETteur
    /** Il est recommandé d'identifier la ressource par une chaîne de caractère ou un nombre conforme à un sytème formel d'identification, ex. ISBN, ISSN, URL */
    QString abeFileGetIdentifier();
    /** Retourne le nom complet du fichier */
    QFileInfo abeFileGetFileName();
    /** Retourne le contenu de l'archive
      * Dans le cas particulier d'un .abb il n'y aura qu'une seule ressource
      * Dans le cas des .abe on pourra faire une boucle
      */
    QFileInfo abeFileGetContent(int num);
    /** Retourne le nom complet du repertoire temporaire dans lequel le fichier est extrait */
    QDir abeFileGetDirectoryTemp();
    /** Retourne la liste des fichiers contenus dans l'archive (initialise quand on ouvre un fichier) */
    QStringList abeFileGetFileList(bool withReservedFileNames=false);
    /** L'entité principalement responsable de la création du contenu de la ressource */
    QString abeFileGetCreator();
    /** La langue du contenu intellectuel de la ressource, fr-fr, en-uk */
    QString abeFileGetLanguage();
    /** Une date associée avec un événement dans le cycle de vie de la ressource au format ISO8601 */
    QString abeFileGetDate();
    /** Typiquement, le sujet sera décrit par un ensemble de mots-clefs ou de phrases ou un code de classification qui précisent le sujet */
    QString abeFileGetSubject();
    /** Typiquement, un titre sera le nom par lequel la ressource est officiellement connue */
    QString abeFileGetTitle();
    /** Typiquement, un élément Droits contiendra un état du droit à gérer une ressource, ou la reférence à un service fournissant cette information.*/
    QString abeFileGetRights();
    /** Une Description peut contenir, mais ce n'est pas limitatif un résumé, une table des matières, une référence à une représentation graphique du contenu, ou un texte libre */
    QString abeFileGetDescription();
    /** Exemples d'Editeurs incluent une personne, une organisation, ou un service. Typiquement, le nom d'une maison d'édition*/
    QString abeFileGetPublisher();
    /** Une entité qui a contribué à la création du contenu*/
    QString abeFileGetContributor();
    /** La liste des mots clés */
    QString abeFileGetKeywords();
    /** Typiquement, Format peut inclure le media ou les dimensions de la ressource, Exemples de dimensions incluent la taille et la durée. Il est recommandé de choisir la valeur du format dans une liste (ex.mime)
      * @see http://www.iana.org/assignments/media-types/index.html
      */
    QString abeFileGetFormat();
    /** La ressource actuelle peut avoir été dérivée d'une autre ressource source, en totalité ou en partie.  */
    QString abeFileGetSource();
    /** Une référence à une autre ressource qui a un rapport avec cette ressource */
    QString abeFileGetRelation();
    /** La portée ou la couverture spatio-temporelle de la ressource. Il est recommandé de choisir la  valeur de Couverture dans un vocabulaire contrôlé (par exemple, un thésaurus de noms géographiques, comme[TGN]) */
    QString abeFileGetCoverage();

    /** Retourne un pointeur sur le fichier LOM */
    AbulEduSCOLOMFrV1 *abeFileGetLOM();

    /** Retourne le type de document
      * @see http://dublincore.org/documents/resource-typelist/
      */
    QString abeFileGetType();

    /**
      * Calcule la somme MD5 du fichier et la retourne
      */
    QString abeFileGetMD5();

    /**
      * Calcule la somme MD5 du fichier passé en paramètre et la retourne
      */
    static QString abeFileGetMD5(const QString otherFileName);

    /**
      * Retourne le fichier sous forme d'une chaine encodée en base64 (utile pour les transferts push/get vers/depuis la médiathèque)
      */
    QString abeFileGetBase64Encoded();

    // ======================================================================== UPDATeur
    void abeFileUpdateLOM();
    void abeFileUpdateDesc();

    // ======================================================================== Divers
    /**
      * Extrait le thumbnail du fichier source vers la destination destination :)
      * Exemple abeExtractIcon("/tmp/toto.abe","/tmp/thumbnails/toto.png")
      */
    static void abeFileExtractIcon(QString source, QString destination);

    /**
      * Retourne directement le thumbnail du fichier
      * Exemple QIcon icon(abuledufile->abeFileGetIcon())
      */
    QPixmap abeFileGetIcon();


    /**
      * Extrait n'importe quel fichier de l'archive et le copie dans destination
      *
      * @param source nom du fichier abe source
      * @param fileSearch nom du fichier qu'on souhaite extraire
      * @param destination nom du fichier complet sous lequel on veut l'exporter
      * @return bool true si l'opération s'est bien passée
      *
      * @code
 if(abeFileExtract("/tmp/toto.abe","Thumbnails/thumbnail.png","/tmp/toto.png")) ...
      * @endcode
      */
    bool abeFileExtract(QString source, QString fileSearch, QString destination);

    /** Permet de dire si on veut ou pas exporter le fichier lom.xml dans le fichier .abe (ne marche que pour le format .abe) */
    void abeFileSetLOMExport(bool value=true);
    void abeFileSetLOM(AbulEduSCOLOMFrV1 *lom);

    /** Nettoie un répertoire (temporaire)
      */
    static bool abeCleanDirectory(QString dirPath, QString dirInitial);

    /** Nettoie le répertoire temporaire du fichier
      */
    bool abeClean();

signals:
    void abeSignalFileOpen(int errcode);

public slots:

private:
    void domAddElement(QString tagName, QString value, QDomElement *parent);
    void fileOpenHandleSpecialFile(QString fileName, QString absoluteFilePath);
    QByteArray makeContentXML();
    QByteArray makeManifestXML(QStringList fileList, QString fileBaseName);
    QByteArray makeManifestRDF();

    bool m_isTemporaryDir; /** Flag pour savoir si on extrait le fichier abe dans un repertoire temporaire (en ce cas on le nettoie a la sortie) ou pas */
    //le petit necessaire pour generer / lire le fichier meta.xml present dans les .zip
    QDomDocument m_domDocument; /** */

    //Le dublin core source traduction http://www-rocq.inria.fr/~vercoust/METADATA/DC-fr.1.1.html
    QString m_language;    /** La langue du contenu intellectuel de la ressource, fr-fr, en-uk */
    QString m_creator;     /** L'entité principalement responsable de la création du contenu de la ressource */
    QString m_date;        /** Une date associée avec un événement dans le cycle de vie de la ressource au format ISO8601 */
    QString m_subject;     /** Typiquement, le sujet sera décrit par un ensemble de mots-clefs ou de phrases ou un code de classification qui précisent le sujet */
    QString m_title;       /** Typiquement, un titre sera le nom par lequel la ressource est officiellement connue */
    QString m_rights;      /** Typiquement, un élément Droits contiendra un état du droit à gérer une ressource, ou la reférence à un service fournissant cette information.*/
    QString m_description; /** Une Description peut contenir, mais ce n'est pas limitatif un résumé, une table des matières, une référence à une représentation graphique du contenu, ou un texte libre */
    QString m_publisher;   /** Exemples d'Editeurs incluent une personne, une organisation, ou un service. Typiquement, le nom d'une maison d'édition */
    QString m_contributor; /** Une entité qui a contribué à la création du contenu */
    QString m_keywords;    /** Typiquement, la liste des mots clés au format suivant: mot1, mot2, mot3 */
    QString m_type;        /** Le type inclut des termes décrivant des catégories, fonctions ou genres généraux pour le contenu, ou des niveaux d'agrégation, Il est recommandé de choisir la valeur du type dans une liste de valeurs
                             * @see http://dublincore.org/documents/resource-typelist/
                             */
    QString m_format;      /** Typiquement, Format peut inclure le media ou les dimensions de la ressource, Exemples de dimensions incluent la taille et la durée. Il est recommandé de choisir la valeur du format dans une liste (ex.mime)
                             * @see http://www.iana.org/assignments/media-types/index.html
                             */
    QString m_identifier;  /** Il est recommandé d'identifier la ressource par une chaîne de caractère ou un nombre conforme à un sytème formel d'identification, ex. ISBN, ISSN, URL */
    QString m_source;      /** La ressource actuelle peut avoir été dérivée d'une autre ressource source, en totalité ou en partie.  */
    QString m_relation;    /** Une référence à une autre ressource qui a un rapport avec cette ressource */
    QString m_coverage;    /** La portée ou la couverture spatio-temporelle de la ressource. Il est recommandé de choisir la  valeur de Couverture dans un vocabulaire contrôlé (par exemple, un thésaurus de noms géographiques, comme[TGN]) */

    QString m_filename;    /** Le nom du fichier tout simplement */
    QString m_fileformat;  /** Le format du fichier (abe ou zip) */
    QString m_fileBase;    /** le chemin de base pour "relativiser" les chemins des fichiers a inclure, par exemple
                             * fileList contient /home/utilisateurs/eric.seigne/leterrier/leterrier-imageo/lesanimaux/data/images/girafe.png
                             * et le logiciel a en fait uniquement besoin d'avoir "lesanimaux/data/images/girafe.png"
                             * donc fileBase vaudrait "/home/utilisateurs/eric.seigne/leterrier/leterrier-imageo/"
                             */
    QDir m_directoryTemp;  /** Un repertoire temporaire dans lequel on extrait le fichier */

    bool     m_exportLOM;  /** Permet de choisir si on veut ou pas export du LOM dans le projet (ne marche qu'avec le format abe)*/
    AbulEduSCOLOMFrV1 *m_lom;    /** Le debut de la gestion LOM dans abuledufilev1 */

    QStringList m_fileList; /** La liste des fichiers contenu dans l'archive */
    QStringList m_reservedFileNames; /** La liste des fichiers réservés */

    QSvgRenderer *m_svgRenderer; /** Permet de gérer le format svg @see resizeImage(..) */

    bool m_localDebug;
};

#endif // ABULEDUFILEV1_H
