/** Classe AbulEduNetworkAccessManagerV1
  * @author 2011-2012 Eric Seigne <eric.seigne@ryxeo.com>
  * @see The GNU Public License (GNU/GPL) v3
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
  * for more details.
  *
  * You should have received a copy of the GNU General Public License along
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  */

#ifndef ABULEDUNETWORKACCESSMANAGERV1_H
#define ABULEDUNETWORKACCESSMANAGERV1_H

#include <QtNetwork/QNetworkAccessManager>
#include <QtNetwork/QNetworkInterface>
#include <QtNetwork/QNetworkRequest>
#include <QtNetwork/QNetworkReply>
#include <QNetworkAccessManager>
#include <QNetworkProxy>
#include <QDebug>
#include <QRegExp>
#include <QStringList>
#include <QUrl>
#include <QHostInfo>
#include <QSharedPointer>
#include <QNetworkRequest>
#include <QEventLoop>
#include <QNetworkReply>
#include <QObject>
#include <QSettings>

#include "maiaObject.h"
#include "abuledunetworkthreadv1.h"
#include "abuleduproxypassworddialog.h"
#include "abuledulocalsocketipcserverv1.h"
#include "abuledulocalsocketipcclientv1.h"

/**
  * Dialogue IPC Client / Serveur
  * HELLO (client) -> serveur, ajoute ce client dans la liste des alertes
  * QUIT (client) -> serveur, supprime ce client de la liste des alertes
  * IPCSSOLOGIN (client) -> serveur, lance une procédure de mise à jour du ticket SSO vers toutes les applications
  * IPCSSOLOGOUT (client) -> serveur, lance la procédure de fermeture de session SSO pour toute les applications
  *
  */

class AbulEduNetworkAccessManagerV1 : public QNetworkAccessManager
{
    Q_OBJECT
    Q_ENUMS(enumAbulEduNetworkAccessManagerProxySettings)
//    Q_PROPERTY(enumAbulEduNetworkAccessManagerProxySettings proxySettings READ abeGetProxySettingsEnum WRITE abeGetProxySettingsEnum USER true)

public:
    enum enumAbulEduNetworkAccessManagerProxySettings {abeProxyDisabled = 0, abeProxyAuto = 1, abeProxyManual = 2, abeProxyAutoURI = 3, abeProxySystemEnv = 4};

    AbulEduNetworkAccessManagerV1( QObject *parent = 0 );
    ~AbulEduNetworkAccessManagerV1();
    QNetworkReply *post(const QNetworkRequest &request, QIODevice *data);
    QNetworkReply *post(const QNetworkRequest &request, const QByteArray &data);
    QNetworkReply *get(const QNetworkRequest &request);

    /** Permet d'affecter le login/pass pour l'authentification reseau
      * Attention, dès qu'on affecte ces valeurs on accroche le signal / slot pour que cet objet
      * gère tout seul l'authentification
      */
    void setAbeLoginPass(QString login, QString pass);

    /** Permet d'ouvrir la session SSO */
    void abeSSOLogin();

    /** Permet de fermer la session SSO */
    void abeSSOLogout();

    /** Permet de savoir si l'authentification SSO est ok
      * @return int: 0 on ne sait pas, 1 ok, -1 erreur
      */
    int abeSSOAuthenticationStatus();

    /**
      * Permet de connecter un slot et de le lancer si l'authentification se passe bien
      * Exemple d'utilisation:
@code
void MainWindow::on_action_Mon_compte_triggered()
{
    if(m_nam->abeSSOAuthenticationStatus() != 1) {
        m_nam->abeOnLoginSuccessGoto(this, SLOT(on_action_Mon_compte_triggered()));
        ui->stackedWidget->setCurrentWidget(ui->pageLogin);
    }
    else {
        ui->stackedWidget->setCurrentWidget(ui->pageIdentite);
    }
}
@endcode
      *
      */
    void abeOnLoginSuccessGoto(QObject *objectDestination, const char* slotDestination);

    /** Permet de connecter un slot et de le lancer si l'authentification est rejetée */
    void abeOnLoginFailureGoto(QObject *objectDestination, const char* slotDestination);

    /** Retourne le login du système d'authentification */
    inline QString abeGetSSOLogin() {return m_login;}

    /** Retourne le mot de passe du système d'authentification */
    inline QString abeGetSSOPassword() {return m_pass;}

    enumAbulEduNetworkAccessManagerProxySettings abeGetProxySettingsEnum() const;
    void abeSetProxySettingsEnum(enumAbulEduNetworkAccessManagerProxySettings settings);

    /** Sauvegarde la configuration proxy */
    void abeSaveProxySettings();

    /** Applique la configuration proxy precedement sauvegardée */
    void abeActivateProxySettings();

    /** Affecte la configuration proxy */
    void abeSetProxyManualSettings(QString serveurHTTP, QString portHTTP);

    /** Autorise uniquement une liste de domaines, cette fonction permet de n'autoriser que quelques domaines.
      * Pratique si on veut faire un logiciel client pour babytwit.fr et qu'on ne souhaite pas que les utilisateurs
      * puissent "partir" sur d'autres sites en suivant des liens (par exemple)
      * @warning: si vous voulez appliquer la liste blanche a un webview il ne faut pas oublier de
      * a) ui->webView->page()->setNetworkAccessManager(m_nam);
      * b) ne pas mettre d'url dans qt-designer: a partir du moment ou un webview a ouvert une page on ne peut plus
      *    changer son networkmanager !
      */
    void abeSetNetworkWhiteList(QStringList domains);

public slots:
    /** Gestion de l'erreur "authentification du proxy" */
    void slotProxyAuthenticationRequired(QNetworkProxy proxy,QAuthenticator* authenticator);

    /** Gestion de l'erreur de telechargement */
    void slotReplyError(QNetworkReply::NetworkError code);

    /** Telechargement et configuration du fichier de configuration proxy automatique */
    void finishedSlotWPAD(QNetworkReply* reply);

    /** Permet d'activer ou d'interdire les popup SSO (actif par défaut)*/
    void abeSSOPopupEnable(bool enable);

    /**
      * Test du proxy (pour verifier s'il est joignable etc.)
      * contexte = update quand on met a jour la conf proxy et qu'on essaye une nouvelle config
      * contexte = start quand on teste le proxy au lancement de l'application
      */
    bool abeProxyConfigTest(QString contexte);

signals:
    /** Permet de connecter un slot si l'authentification SSO n'est pas bonne */
    void ssoAuthStatus(int ssoAuthErrorValue);

    /** Permet d'afficher une popup par exemple si propagation de l'authentification via alacarte */
    void ssoAuthPropagated(QString login);

protected:
    /**
      * permet d'appliquer le filtre de la liste blanche :-)
      */
    QNetworkReply *createRequest( Operation op, const QNetworkRequest &req, QIODevice *outgoingData );

private slots:
    /** Gère l'authentification SSO */
    void slotSSOAuth(QNetworkReply* networkReply,QAuthenticator* authenticator);
    void slotFinishedSSOLogin(QNetworkReply* reply);
    void slotFinishedSSOLogout(QNetworkReply* reply);
    void slotSSOPopupClosed();
    void slotLocalSocketMessageReceived(QString message);
    /** Tentative de gestion des 'erreurs' ssl pour insérer nos certificats -> solution dans le constructeur
      * du manager on ajoute les certificats de manière globale
      */
    void slotSslErrors ( QNetworkReply * reply, const QList<QSslError> & errors );

    void slotReplyProxyNoError();


private:
    void clear();
    void p_ipcSSOLogin(QStringList params);

    /**
      * Configuration du proxy : pas de proxy
      */
    bool p_proxyConfigNoProxy();

    /**
      * Configuration du proxy a partir de l'environnement systeme par exemple
      * http_proxy=http://192.168.0.1:3128/
      */
    bool p_proxyConfigFromSystemEnv();

    /**
      * Configuration du proxy a partir de du protocole de configuration automatique WPAD
      * http://en.wikipedia.org/wiki/Web_Proxy_Autodiscovery_Protocol
      */
    bool p_proxyConfigFromWPAD();

    /**
      * Configuration du proxy a partir d'une URI de configuration automatique
      * exemple http://servecole/proxy/config.pac
      */
    bool p_proxyConfigFromURI();

    /**
      * Configuration proxy manuelle
      */
    bool p_proxyConfigManual();

    //    p_ipcSSOLogout(QStringList params);

private:
    QPointer<MaiaObject>               m_maiaObject;
//    QSharedPointer<QAuthenticator>     m_ssoAuthenticator;
    QAuthenticator                    *m_ssoAuthenticator;

    bool m_localDebug;
    int                                m_proxyPassAsked;   //Le nombre de fois ou on a demande le mot de passe proxy
    QNetworkReply*                     m_networkReply;     //Le reply qui permet de relancer la connexion sur
                                                           //la même page en cas de capture proxy par ex
    int                                m_ssoAuthentifiedStatus;//un flag pour savoir si oui ou non on est authentifie: 0 on ne sait pas, 1 ok, -1 erreur
    int                                m_passAsked;        //Le nombre de fois ou on a demande le mot de passe sso
    QStringList                        m_networkWhiteList; //La liste des domaines DNS autorisés (si whitelist)

    bool    m_ssoInProgress; //pour savoir si on est en train de s'authentifier, évite d'avoir les connexions en //
    bool    m_ssoPopupIsShow;
    bool    m_ssoUsePopup;     //permet de désactiver totalement les popup, par ex. dans alacarte on gere le sso dans un widget intégré
    QString m_login;
    QString m_pass;
    QString *m_proxyLogin;
    QString *m_proxyPass;

    AbulEduLocalSocketIpcServerV1    *m_localSocketServer; //Un serveur pour passer le ticket SSO aux clients
    AbulEduLocalSocketIpcClientV1    *m_localSocketClient; //Un client pour passer le ticket SSO aux clients

    //Pour le système d'aiguillage automatique du système d'authentification
    const char                    *m_destinationOnLoginSuccess;       //ou on va apres etre authentifie ?
    QObject                       *m_destinationOnLoginSuccessObject; //et de quel objet ?
    const char                    *m_destinationOnLoginFailure;       //ou on va apres etre authentifie ?
    QObject                       *m_destinationOnLoginFailureObject; //et de quel objet ?

    enumAbulEduNetworkAccessManagerProxySettings m_proxySettingsEnum; //Le type de proxy
};

#endif // ABULEDUNETWORKACCESSMANAGERV1_H
